package com.vp.plugin.sample.retrieveuserstoryscenario.actions;

import com.vp.plugin.ApplicationManager;
import com.vp.plugin.action.VPAction;
import com.vp.plugin.action.VPActionController;
import com.vp.plugin.model.IExtension;
import com.vp.plugin.model.IModelElement;
import com.vp.plugin.model.IProject;
import com.vp.plugin.model.IStep;
import com.vp.plugin.model.IStepContainer;
import com.vp.plugin.model.IUseCase;
import com.vp.plugin.model.IUserStory;
import com.vp.plugin.model.factory.IModelElementFactory;

public class RetrieveUserStoryScenarioActionControl implements VPActionController {

	@Override
	public void performAction(VPAction arg0) {
		// Obtain the current opening project
		IProject project = ApplicationManager.instance().getProjectManager().getProject();		
		// Retrieve all use cases in the project
		IModelElement[] useCases = project.toModelElementArray(IModelElementFactory.MODEL_TYPE_USE_CASE);

		if (useCases != null && useCases.length > 0) {
			// Walk through all the use cases
			for (int i = 0; i < useCases.length; i++) {
				IUseCase useCase = (IUseCase) useCases[i];
				// Print out the name of the use case
				ApplicationManager.instance().getViewManager().showMessage("Use Case: " + useCase.getName());
				// Retrieve all user stories from use case
				IUserStory[] userStories = useCase.toUserStoryArray();
				if (userStories != null && userStories.length > 0) {
					// Walk through all user stories
					for (int j = 0; j < userStories.length; j++) {					
						IUserStory userStory = (IUserStory) userStories[j];
						// Retrieve all scenarios from user story
						IStepContainer[] scenarios = userStory.toScenarioArray();
						if (scenarios != null && scenarios.length > 0) {
							// Walk though all scenarios
							for (int k = 0; k < scenarios.length; k++) {
								IStepContainer scenario = (IStepContainer) scenarios[k];
								// Print put the name of the scenario
								ApplicationManager.instance().getViewManager().showMessage("Scenario: " + scenario.getName());
								// Retrieve all steps from scenario
								IStep[] steps = scenario.toStepArray();
								if (steps != null && steps.length > 0) {
									for (int l = 0 ; l < steps.length; l++) {
										// Walk through the steps
										printStep((IStep) steps[l], "" + (l+1), 0);									
									}
								}
							}							
						}						
					}
				}				
			}			
		}
	}
	
	private void printStep(IStep step, String parentIndex, int index) {

		StringBuffer sb = new StringBuffer();
		sb.append(parentIndex);
		if (index != 0) {
			sb.append(".");
			sb.append(index);				
			
			parentIndex = parentIndex + "." + index;
		}
		sb.append(": ");
		sb.append(step.getName());
		ApplicationManager.instance().getViewManager().showMessage(sb.toString());
		
		// Retrieve the extension steps
		IExtension[] extensions = step.toExtensionArray();
		if (extensions != null && extensions.length > 0) {
			// Walk through the extension steps
			for (int i = 0; i < extensions.length; i++) {
				IExtension extension = (IExtension) extensions[i];
				ApplicationManager.instance().getViewManager().showMessage("Extension: " + extension.getName());														
			}
		}
		// Retrieve the sub-steps
		IStep[] steps = step.toStepArray();
		if (steps != null && steps.length > 0) {
			// Walk through the sub-steps
			for (int i = 0; i < steps.length; i++) {
				printStep(steps[i], parentIndex, i+1);
			}
		}
		
	}

	@Override
	public void update(VPAction arg0) {
		// TODO Auto-generated method stub
		
	}

}
